@@ -6,6 +6,7 @@ use 5.008_001;
 
 use Carp;
 use IO::Socket;
+use Time::HiRes;
 
 use App::Memcached::CLI::Util qw(is_unixsocket debug);
 
@@ -47,14 +48,13 @@ sub _retrieve {
     my $self = shift;
     my ($cmd, $keys) = @_;
 
-    my $socket = $self->{socket};
     my $key_str = join(q{ }, @$keys);
-    print $socket "$cmd $key_str\r\n";
+    $self->{socket}->write("$cmd $key_str\r\n");
 
     my @results;
 
     while (1) {
-        my $response = <$socket>;
+        my $response = $self->_readline;
         next if ($response =~ m/^[\r\n]+$/);
         if ($response =~ m/^VALUE (\S+) (\d+) (\d+)(?: (\d+))?/) {
             my %data = (
@@ -63,7 +63,10 @@ sub _retrieve {
                 length => $3,
                 cas    => $4,
             );
-            read $socket, $response, $data{length};
+            local $SIG{ALRM} = sub { die 'Timed out to Read Socket.' };
+            alarm 3;
+            $self->{socket}->read($response, $data{length});
+            alarm 0;
             $data{value} = $response;
             push @results, \%data;
         } elsif ($response =~ m/^END/) {
@@ -102,10 +105,14 @@ sub _store {
         return length $value;
     }->();
 
-    my $socket = $self->{socket};
-    print $socket "$cmd $key $flags $expire $bytes\r\n";
-    print $socket "$value\r\n";
-    my $response = $self->_readline;
+    $self->{socket}->write("$cmd $key $flags $expire $bytes\r\n");
+    $self->{socket}->write("$value\r\n");
+    my $response = eval {
+        return $self->_readline;
+    };
+    if ($@) {
+        confess qq{Failed to store data by "$cmd"! ($key, $value) ERROR: } . $@;
+    }
     if ($response !~ m/^STORED/) {
         debug qq{Failed to $cmd data as ($key, $value)};
         return;
@@ -127,10 +134,14 @@ sub cas {
         return length $value;
     }->();
 
-    my $socket = $self->{socket};
-    print $socket "cas $key $flags $expire $bytes $cas\r\n";
-    print $socket "$value\r\n";
-    my $response = $self->_readline;
+    $self->{socket}->write("cas $key $flags $expire $bytes $cas\r\n");
+    $self->{socket}->write("$value\r\n");
+    my $response = eval {
+        return $self->_readline;
+    };
+    if ($@) {
+        confess qq{Failed to store data by "cas"! ($key, $value) ERROR: } . $@;
+    }
     if ($response !~ m/^STORED/) {
         debug qq{Failed to set data as ($key, $value) with cas $cas};
         return;
@@ -142,9 +153,7 @@ sub delete {
     my $self = shift;
     my $key  = shift;
 
-    my $socket = $self->{socket};
-    print $socket "delete $key\r\n";
-    my $response = $self->_readline;
+    my $response = $self->query_one("delete $key");
     if ($response !~ m/^DELETED/) {
         warn "Failed to delete '$key'";
         return;
@@ -157,9 +166,7 @@ sub touch {
     my $key    = shift;
     my $expire = shift;
 
-    my $socket = $self->{socket};
-    print $socket "touch $key $expire\r\n";
-    my $response = $self->_readline;
+    my $response = $self->query_one("touch $key $expire");
     if ($response =~ m/^NOT_FOUND/) {
         debug "No such data KEY '$key'";
         return;
@@ -179,9 +186,7 @@ sub _incr_decr {
     my $key    = shift;
     my $number = shift;
 
-    my $socket = $self->{socket};
-    print $socket "$cmd $key $number\r\n";
-    my $response = $self->_readline;
+    my $response = $self->query_one("$cmd $key $number");
     if ($response =~ m/^NOT_FOUND/) {
         warn "No such data KEY '$key'";
         return;
@@ -193,17 +198,62 @@ sub _incr_decr {
     return $new_value;
 }
 
-sub version {
+sub query_one {
     my $self  = shift;
     my $query = shift;
 
-    my $socket = $self->{socket};
-    print $socket "version\r\n";
-    my $response = $self->_readline;
+    $self->{socket}->write("$query\r\n");
+    my $response = eval {
+        return $self->_readline;
+    };
+    if ($@) {
+        confess "Failed to query! query: $query ERROR: " . $@;
+    }
     chomp $response;
     return $response;
 }
 
+sub query_any {
+    my $self  = shift;
+    my $query = shift;
+
+    $self->{socket}->write("$query\r\n");
+
+    # Save blocking mode
+    my $blocking_mode = $self->{socket}->blocking;
+
+    my $response = eval {
+        local $SIG{ALRM} = sub { die 'Timed out to Read Socket.' };
+        alarm 5;
+        my $resp = q{};
+        $self->{socket}->blocking(0);
+        my $getline_from_sock = sub {
+            for my $i (1..3) {
+                my $line = $self->{socket}->getline;
+                return $line if defined $line;
+                #debug "failed to getline - $i. query: $query";
+                Time::HiRes::sleep(0.01);
+            }
+            return;
+        };
+        while (my $line = $getline_from_sock->()) {
+            $resp .= $line;
+        }
+        alarm 0;
+        return $resp;
+    };
+    my $err = $@;
+
+    # Restore blocking mode
+    $self->{socket}->blocking($blocking_mode);
+
+    if ($err) {
+        confess "Failed to query! query: $query ERROR: " . $err;
+    }
+
+    return $response;
+}
+
 sub query {
     my $self  = shift;
     my $query = shift;
@@ -220,8 +270,7 @@ sub _query {
     my $self  = shift;
     my $query = shift;
 
-    my $socket = $self->{socket};
-    print $socket "$query\r\n";
+    $self->{socket}->write("$query\r\n");
 
     my @response;
     while (1) {
@@ -236,11 +285,10 @@ sub _query {
 }
 
 sub _readline {
-    my $self   = shift;
-    my $socket = $self->{socket};
+    my $self = shift;
     local $SIG{ALRM} = sub { die 'Timed out to Read Socket.' };
     alarm 3;
-    my $line = <$socket>;
+    my $line = $self->{socket}->getline;
     alarm 0;
     return $line;
 }
