@@ -17,7 +17,7 @@ use App::Memcached::CLI::Help;
 use App::Memcached::CLI::Item;
 use App::Memcached::CLI::Util ':all';
 
-use version; our $VERSION = 'v0.8.0';
+use version; our $VERSION = 'v0.9.0';
 
 my $PROGRAM = basename $0;
 
@@ -44,6 +44,7 @@ my %COMMAND2ALIASES = (
     touch      => [],
     delete     => [],
     flush_all  => [qw(flush)],
+    call       => [],
 );
 my %COMMAND_OF;
 while (my ($cmd, $aliases) = each %COMMAND2ALIASES) {
@@ -221,6 +222,18 @@ sub _sorted_aliases_of {
     return ($command);
 }
 
+sub call {
+    my $self = shift;
+    my @commands = @_;
+    unless (@commands) {
+        print "No COMMAND given.\n";
+        return;
+    }
+    my $response = $self->{ds}->query_any(join q{ }, @commands);
+    print $response;
+    return 1;
+}
+
 sub get {
     my $self = shift;
     return $self->_retrieve('get', @_);
@@ -369,7 +382,7 @@ sub delete {
 
 sub version {
     my $self = shift;
-    my $version = $self->{ds}->version;
+    my $version = $self->{ds}->query_one('version');
     print "$version\n";
     return 1;
 }
@@ -503,13 +516,16 @@ sub detail {
         print "Mode must be 'on' or 'off'!\n";
         return;
     }
-    my $response = $self->{ds}->query("stats detail $mode");
-    print "$_\n" for @$response;
+    my $response = $self->{ds}->query_one("stats detail $mode");
     my %result = (
         on  => 'Enabled',
         off => 'Disabled',
     );
-    print "$result{$mode} stats collection for detail dump.\n";
+    if ($response !~ m/^OK/) {
+        print "Seems failed to set detail $mode\n";
+        return;
+    }
+    print "OK - $result{$mode} stats collection for detail dump.\n";
     return 1;
 }
 
@@ -518,8 +534,8 @@ sub flush_all {
     my $delay = shift;
     my $query = 'flush_all';
     if ($delay) { $query .= " $delay"; }
-    my $response = $self->{ds}->query($query);
-    print "OK\n";
+    my $response = $self->{ds}->query_one($query);
+    print "$response\n";
     return 1;
 }
 
